#!/bin/bash

sys=`cat /etc/os-release | grep NAME`
ubuntu=`echo $sys | grep Ubuntu`
centos=`echo $sys | grep CentOS`
debian=`echo $sys | grep Debian`
fedora=`echo $sys | grep Fedora`

if [ "$centos" ] || [ "$fedora" ]; then
        yum -y install perl-devel perl-ExtUtils-CBuilder perl-ExtUtils-MakeMaker patchutils patch
        xx=`which gcc`
        xx=`echo $xx | grep gcc`
        if [ ! $xx ]; then
                        yum install gcc
        fi
        xx=`which make`
  xx=`echo $xx | grep make`
  if [ ! $xx ]; then
                        yum install make
  fi
  xx=`which unzip`
        xx=`echo $xx | grep unzip`
        if [ ! $xx ]; then
                        yum install unzip
        fi

elif [ "$ubuntu" ] || [ "$debian" ]; then
        apt-get install libproc-processtable-perl
        apt-get install libelf-dev
        xx=`which patch`
  xx=`echo $xx | grep patch`
  if [ ! $xx ]; then
                        apt-get install patchutils patch
  fi

        xx=`which gcc`
  xx=`echo $xx | grep gcc`
  if [ ! $xx ]; then
                        apt-get install gcc
  fi

  xx=`which make`
  xx=`echo $xx | grep make`
  if [ ! $xx ]; then
                        apt-get install make
  fi

fi

kernelv6=(
"v6.9_assign_str.patch"
"v6.9_i2c_mux.patch"
"v6.9_i2c_mux_tbs.patch"
"v6.8_spi.patch"
"v6.8_ccs.patch"
"v6.7_cec.patch"
"v6.5_dvb_math.patch"
"v6.5_pcm_copy.patch"
"v6.4_pin_user_pages.patch"
"v6.3_check_endpoint.patch"
"v6.3_class_create.patch"
"v6.2_class.patch"
"v6.2_probe.patch"
"v6.2_probe_tbs.patch"
"v6.2_remove.patch"
"v6.2_v4l2-core.patch"
"v6.1_class.patch"
"v6.1_i2c_probe.patch"
"v6.1_ipu3.patch"
"v6.1_timer.patch"
"v6.1_videobuf2.patch"
"v6.0_i2c_remove.patch"
"v6.0_i2c_remove_tbs.patch"
"v6.0_pt3.patch"
"v6.0_testdrv.patch"
"v6.0_v4l2-tpg.patch"
)

kernelv5=(
"v5.18_misc.patch"
"v5.18_rc.patch"
"v5.18_rc2.patch"
"v5.18_v4l2.patch"
"v5.17_iosys.patch"
"v5.17_overflow.patch"
"v5.17_spi.patch"
"v5.15_container_of.patch"
"v5.14_bus_void_return.patch"
"v5.13_stk1160.patch"
"v5.12_uvc.patch"
"v5.11_isa.patch"
"v5.10_vb2_dma_buf_map.patch"
"v5.9_netup_unidvb_devm_revert.patch"
"v5.9_tasklet.patch"
"v5.7_vm_map_ram.patch"
"v5.7_pin_user_pages.patch"
"v5.7_mmap_read_lock.patch"
"v5.7_define_seq_attribute.patch"
"v5.6_pm_runtime_get_if_active.patch"
"v5.6_const_fb_ops.patch"
"v5.6_pin_user_pages.patch"
"v5.5_alsa_pcm_api_updates.patch"
"v5.5_dev_printk_h.patch"
"v5.5_memtype_h.patch"
"v5.5_vb2_kmap.patch"
"v5.4_revert_spi_transfer.patch"
"v5.4_async.patch"
"v5.3_device_match_fwnode.patch"
"v5.3_dma_resv.patch"
"v5.1_devm_i2c_new_dummy_device.patch"
"v5.1_vm_map_pages.patch"
"v5.0_time32.patch"
"v5.0_ipu3-cio2.patch"
"v5.0_saa7146.patch"
"v5.0_gpio.patch"
)

kernelv4=(
"v4.20_access_ok.patch"
"v4.18_vb2_map_atomic.patch"
"v4.18_fwnode_args_args.patch"	
"v4.18_ccs_bitops.patch"
"v4.18_ida_simple.patch"
"v4.17_i2c_check_num_msgs.patch"
"v4.16_poll_requested_events.patch"
"v4.15_pmdown_time.patch"
"v4.15_async.patch"
"v4.14_fwnode_handle_get.patch"
"v4.14_saa7146_timer_cast.patch"
"v4.14_module_param_call.patch"
"v4.13_remove_nospec_h.patch"
"v4.13_fwnode_graph_get_port_parent.patch"
"v4.13_drmP.patch"
)   



cd linux
patch -s -f -N -p1 -i ../backports/api_version.patch
patch -s -f -N -p1 -i ../backports/pr_fmt.patch
patch -s -f -N -p1 -i ../backports/debug.patch
patch -s -f -N -p1 -i ../backports/drx39xxj.patch
patch -s -f -N -p1 -i ../backports/ccs.patch
patch -s -f -N -p1 -i ../backports/rc-cec.patch
patch -s -f -N -p1 -i ../backports/no_atomic_include.patch

ver=`uname -r`
echo "Applying patches for kernel $ver"

v1=`echo $ver | cut -d "." -f 1`
v2=`echo $ver | cut -d "." -f 2`
v3=`echo $ver | cut -d "." -f 3 | cut -d "-" -f 1`
echo "$v1 $v2 $v3"

if [ $v1 -eq 6 ]; then
		len6=${#kernelv6[@]}
		#echo $len6
		index=0
		while(( $index<$len6 ))
		do
			v11=`echo ${kernelv6[$index]} | cut -d "." -f 1 |  tr -d "a-zA-Z" `
			v22=`echo ${kernelv6[$index]} | cut -d "." -f 2 |  cut -d "_" -f 1`		
						
			if [ $v1 -eq $v11 -a $v2 -le $v22 ]; then
					#echo "$v11  $v22"
					patch -s -f -N -p1 -i ../backports/${kernelv6[$index]}
					#echo "${kernelv6[$index]}"
			else
					break
			fi
			let "index++"
		done
elif [ $v1 -eq 5 ]; then
		#V6.xx.xx
		len6=${#kernelv6[@]}
		index=0
		while(( $index<$len6 ))
		do
			patch -s -f -N -p1 -i ../backports/${kernelv6[$index]}
			let "index++"
		done
		
		len5=${#kernelv5[@]}
		#echo $len5
		index=0
		while(( $index<$len5 ))
		do
			v11=`echo ${kernelv5[$index]} | cut -d "." -f 1 |  tr -d "a-zA-Z" `
			v22=`echo ${kernelv5[$index]} | cut -d "." -f 2 |  cut -d "_" -f 1`		
						
			if [ $v1 -eq $v11 -a $v2 -le $v22 ]; then
					#echo "$v11  $v22"
					patch -s -f -N -p1 -i ../backports/${kernelv5[$index]}
					#echo "${kernelv5[$index]}"
			else
					break
			fi
			let "index++"
		done	
elif [ $v1 -eq 4 ]; then
		#V6.xx.xx
		len6=${#kernelv6[@]}
		index=0
		while(( $index<$len6 ))
		do
			patch -s -f -N -p1 -i ../backports/${kernelv6[$index]}
			let "index++"
		done

		#V5.xx.xx
		len5=${#kernelv5[@]}
		index=0
		while(( $index<$len5 ))
		do
			patch -s -f -N -p1 -i ../backports/${kernelv5[$index]}
			let "index++"
		done

		len4=${#kernelv4[@]}
		#echo $len4
		index=0
		while(( $index<$len4 ))
		do
			#echo ${kernelv4[$index]}
			v11=`echo ${kernelv4[$index]} | cut -d "." -f 1 |  tr -d "a-zA-Z" `
			v22=`echo ${kernelv4[$index]} | cut -d "." -f 2 | cut -d "_" -f 1`		
						
			if [ $v1 -eq $v11 -a $v2 -le $v22 ]; then
					#echo "$v11  $v22"
					patch -s -f -N -p1 -i ../backports/${kernelv4[$index]}
					#echo "${kernelv4[$index]}"
			else
					break
			fi
			let "index++"
		done		
						
fi

cd ../
if [ -f tbs-tuner-firmwares*.tar.bz2 ]; then
		tar jxvf tbs-tuner-firmwares*.tar.bz2 -C /lib/firmware/
else
		echo "firmware file not exist!!!"		
fi

